// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'bytertc_event_define.dart';

/// Authentication method for speech recognition services. Please consult technical support for details.
enum ASRAuthorizationType {
  /// Token
  token,

  /// Signature
  signature,
}

/// Verification information required when using ASR services
class RTCASRConfig {
  /// User ID
  String uid;

  /// Access token
  String accessToken;

  /// Private key
  ///
  /// This value cannot be empty in Signature authentication mode and should be empty in token authentication mode.
  String secretKey;

  /// Authentication method
  ASRAuthorizationType authorizationType;

  /// Scenario information
  String cluster;

  /// APP ID
  String appId;

  /// @nodoc
  RTCASRConfig({
    required this.uid,
    required this.accessToken,
    this.secretKey = '',
    this.authorizationType = ASRAuthorizationType.token,
    required this.cluster,
    required this.appId,
  });

  /// @nodoc
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'uid': uid,
      'accessToken': accessToken,
      'secretKey': secretKey,
      'authorizationType': authorizationType.index,
      'cluster': cluster,
      'appId': appId,
    };
  }
}

/// [message]: The recognized text message.
typedef OnMessageType = void Function(String message);

/// [errorCode]: Error code
/// + `<0`: Invalid parameter or wrong API call order.
/// + `>0`: Other errors.
///
/// [errorMessage]: Reasons for errors.
typedef OnErrorMsgType = void Function(int errorCode, String errorMessage);

/// @nodoc('Not available')
/// Speech recognition service usage status callback
class RTCASREngineEventHandler {
  /// Successfully enabled.
  EmptyCallbackType? onSuccess;

  /// Callback for successful voice to text conversion. All the recognized text is returned through this callback.
  ///
  /// If the network connection is interrupted during the recognition process, the callback after the reconnection carries only the text message recognized after the reconnection.
  OnMessageType? onMessage;

  /// Callback for internal error.
  OnErrorMsgType? onErrorMsg;

  /// @nodoc
  RTCASREngineEventHandler({
    this.onSuccess,
    this.onMessage,
    this.onErrorMsg,
  });
}
