// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'bytertc_video_effect_api.dart';

/// Rectangle area.
class Rectangle {
  /// The x coordinate of the upper left corner of the rectangular area.
  final int? x;

  /// The y coordinate of the upper left corner of the rectangular area.
  final int? y;

  /// Rectangle width in px.
  final int? width;

  /// Rectangle height in px.
  final int? height;

  /// @nodoc
  const Rectangle({
    this.x,
    this.y,
    this.width,
    this.height,
  });

  /// @nodoc
  factory Rectangle.fromMap(Map<dynamic, dynamic> map) {
    return Rectangle(
      x: map['x'],
      y: map['y'],
      width: map['width'],
      height: map['height'],
    );
  }
}

/// Face detection result.
class FaceDetectionResult {
  /// Face detection result:
  /// + 0: Success.
  /// + !0: Failure, see [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table) for the reason.
  final int? detectResult;

  /// Width of the original image (px).
  final int? imageWidth;

  /// Height of the original image (px).
  final int? imageHeight;

  /// The face recognition rectangles. The length of the array is the same as the number of detected faces.
  final List<Rectangle>? faces;

  /// The time stamp of the video frame using face detection.
  final int? frameTimestampUs;

  /// @nodoc
  const FaceDetectionResult({
    this.detectResult,
    this.imageWidth,
    this.imageHeight,
    this.faces,
    this.frameTimestampUs,
  });

  /// @nodoc
  factory FaceDetectionResult.fromMap(Map<dynamic, dynamic> map) {
    return FaceDetectionResult(
      detectResult: map['detectResult'],
      imageWidth: map['imageWidth'],
      imageHeight: map['imageHeight'],
      frameTimestampUs: map['frameTimestampUs'],
      faces: (map['faces'] as List<dynamic>)
          .map((e) => Rectangle.fromMap(e))
          .toList(),
    );
  }
}

/// [result]: Face detection results.
typedef OnFaceDetectResultType = void Function(FaceDetectionResult result);

/// Face detection observer
class RTCFaceDetectionObserver {
  /// Callback of the face detection result with Effect SDK.
  ///
  /// After registering an [RTCFaceDetectionObserver] by calling [RTCVideoEffect.enableFaceDetection] and using the Effect SDK integrated in the RTC SDK, you will receive this callback.
  OnFaceDetectResultType? onFaceDetectResult;

  /// @nodoc
  RTCFaceDetectionObserver({
    this.onFaceDetectResult,
  });
}
