// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'dart:typed_data';

import '../src/bytertc_video_event_impl.dart';
import 'bytertc_audio_defines.dart';
import 'bytertc_audio_mixing_api.dart';
import 'bytertc_event_define.dart';
import 'bytertc_media_defines.dart';
import 'bytertc_media_player_api.dart';
import 'bytertc_rts_defines.dart';
import 'bytertc_video_api.dart';
import 'bytertc_video_defines.dart';

/// [code]: Warning code.
typedef OnWarningType = void Function(WarningCode code);

/// [code]: Error code.
typedef OnErrorType = void Function(ErrorCode code);

/// [extensionName]: The name of extension.
///
/// [msg]: Error message.
typedef OnExtensionAccessErrorType = void Function(
    String extensionName, String msg);

/// [roomId]: The ID of the room where the error occurred.
///
/// [errorCode]: Possible errors when creating a room.
typedef OnCreateRoomStateChangedType = void Function(
    String roomId, int errorCode);

/// [state]: Current connection status between SDK and signaling server.
typedef OnConnectionStateChanged = void Function(RTCConnectionState state);

/// [type]: Current network connection type.
typedef OnNetworkTypeChangedType = void Function(NetworkType type);

/// [roomId]: The ID of the room where the remote user who enables/disables media capturing is in.
///
/// [uid]: The ID of the remote user who enables/disables media capturing.
typedef OnUserOperateMediaCaptureType = void Function(
    String roomId, String uid);

/// [index]: Type of the audio stream.
typedef OnFirstLocalAudioFrameType = void Function(StreamIndex index);

/// [streamKey]: Information about the remote audio stream.
typedef OnFirstRemoteAudioFrameType = void Function(RemoteStreamKey streamKey);

/// [audioPropertiesInfos]: Local audio information.
typedef OnLocalAudioPropertiesReportType = void Function(
    List<LocalAudioPropertiesInfo> audioPropertiesInfos);

/// [audioPropertiesInfos]: Remote audio information.
///
/// [totalRemoteVolume]: Total volume of all subscribed remote streams.
typedef OnRemoteAudioPropertiesReportType = void Function(
    List<RemoteAudioPropertiesInfo> audioPropertiesInfos,
    int totalRemoteVolume);

/// [roomId]: ID of the room where the ActiveSpeaker stays
///
/// [uid]: ID of the ActiveSpeaker
typedef OnActiveSpeakerType = void Function(String roomId, String uid);

/// [streamIndex]: Stream type.
///
/// [videoFrameInfo]: Video information.
typedef OnFirstLocalVideoFrameCapturedType = void Function(
    StreamIndex streamIndex, VideoFrameInfo videoFrameInfo);

/// [streamKey]: Information about the remote stream.
///
/// [videoFrameInfo]: Information about the video frame.
typedef OnFirstRemoteVideoFrameRenderedType = void Function(
    RemoteStreamKey streamKey, VideoFrameInfo videoFrameInfo);

/// [streamKey]: Remote stream information.
///
/// [videoFrameInfo]: Video frame information.
typedef OnRemoteVideoSizeChangedType = void Function(
    RemoteStreamKey streamKey, VideoFrameInfo videoFrameInfo);

/// [streamIndex]: Stream type.
///
/// [videoFrameInfo]: Video frame information.
typedef OnLocalVideoSizeChangedType = void Function(
    StreamIndex streamIndex, VideoFrameInfo videoFrameInfo);

/// [deviceId]: Device ID
///
/// [deviceType]: Device type
///
/// [deviceState]: Device status
///
/// [deviceError]: Device related error
typedef OnAudioDeviceStateChangedType = void Function(
    String deviceId,
    AudioDeviceType deviceType,
    MediaDeviceState deviceState,
    MediaDeviceError deviceError);

/// [deviceId]: Device ID
///
/// [deviceType]: Device type
///
/// [deviceState]: Device status
///
/// [deviceError]: Device related error
typedef OnVideoDeviceStateChangedType = void Function(
    String deviceId,
    VideoDeviceType deviceType,
    MediaDeviceState deviceState,
    MediaDeviceError deviceError);

/// [deviceId]: Device ID
///
/// [deviceType]: Device type
///
/// [deviceWarning]: Device related warning code
typedef OnAudioDeviceWarningType = void Function(String deviceId,
    AudioDeviceType deviceType, MediaDeviceWarning deviceWarning);

/// [deviceId]: Device ID
///
/// [deviceType]: Device type
///
/// [deviceWarning]: Device related warning code
typedef OnVideoDeviceWarningType = void Function(String deviceId,
    VideoDeviceType deviceType, MediaDeviceWarning deviceWarning);

/// [roomId]: ID of the room from which the stream is published.
///
/// [userInfo]: User information.
///
/// [state]: First frame sent status.
typedef OnMediaFrameSendStateChangedType = void Function(
    String roomId, UserInfo userInfo, FirstFrameSendState state);

/// [roomId]: ID of the room which the stream publisher is in.
///
/// [userInfo]: User information.
///
/// [state]: First frame playback status.
typedef OnMediaFramePlayStateChangedType = void Function(
    String roomId, UserInfo userInfo, FirstFramePlayState state);

/// [route]: New audio playback routing
typedef OnAudioRouteChangedType = void Function(AudioRoute route);

/// [streamKey]: Including the SEI sender's username, room name and media stream.
///
/// [message]: Content of the SEI message received.
typedef OnSEIMessageReceivedType = void Function(
    RemoteStreamKey streamKey, Uint8List message);

/// [streamKey]: Remote stream information.
///
/// [event]: Status of the black frame video stream.
typedef OnSEIStreamUpdateType = void Function(
    RemoteStreamKey streamKey, SEIStreamUpdateEvent event);

/// [streamKey]: Remote stream information.
///
/// [streamType]: Media stream type.
///
/// [data]: Message content.
typedef OnStreamSyncInfoReceivedType = void Function(
    RemoteStreamKey streamKey, SyncInfoStreamType streamType, Uint8List data);

/// [state]: Status of local audio device.
///
/// [error]: Error code when the state of the local audio stream changes.
typedef OnLocalAudioStateChangedType = void Function(
    LocalAudioStreamState state, LocalAudioStreamError error);

/// @nodoc('Not available')
/// [streamKey]: Remote stream information.
///
/// [state]: Remote audio stream status.
///
/// [reason]: Reasons for change in remote audio stream status.
typedef OnRemoteAudioStateChangedType = void Function(RemoteStreamKey streamKey,
    RemoteAudioState state, RemoteAudioStateChangeReason reason);

/// @nodoc('Not available')
/// [index]: Stream type.
///
/// [state]: Local video stream status.
///
/// [error]: Error code when local video status changes.
typedef OnLocalVideoStateChangedType = void Function(StreamIndex index,
    LocalVideoStreamState state, LocalVideoStreamError error);

/// @nodoc('Not available')
/// [streamKey]: Information about the remote video stream.
///
/// [state]: Remote video stream status.
///
/// [reason]: The reason for the change in the status of the remote video stream.
typedef OnRemoteVideoStateChangedType = void Function(RemoteStreamKey streamKey,
    RemoteVideoState state, RemoteVideoStateChangeReason reason);

/// @nodoc('Not available')
typedef OnRemoteVideoSuperResolutionModeChangedType = void Function(
    RemoteStreamKey streamKey,
    VideoSuperResolutionMode mode,
    VideoSuperResolutionModeChangedReason reason);

/// @nodoc('Not available')
typedef OnVideoDenoiseModeChangedType = void Function(
    VideoDenoiseMode mode, VideoDenoiseModeChangedReason reason);

/// [uid]: ID of the logged-in user.
///
/// [errorCode]: Login result.
///
/// [elapsed]: The time taken from calling [RTCVideo.login] to the result being returned.
typedef OnLoginResultType = void Function(
    String uid, LoginErrorCode errorCode, int elapsed);

/// [reason]: The reason why users logout.
typedef OnLogoutType = void Function(LogoutReason reason);

/// [error]: Setting result
/// + 200: Success.
/// + Others: Failure, see [UserMessageSendResult] for details.
typedef OnServerParamsSetResultType = void Function(int error);

/// [peerUid]: ID of the user to be queried.
///
/// [status]: Login status of the queried user.
typedef OnGetPeerOnlineStatusType = void Function(
    String peerUid, UserOnlineStatus status);

/// [msgid]: ID of this message.
///
/// [error]: Sending result.
///
/// [message]: Information returned by the application server in the ACK after receiving an HTTP request. The message should not exceed 64 KB.
typedef OnServerMessageSendResultType = void Function(
    int msgid, UserMessageSendResult error, Uint8List message);

/// [type]: Indicating the network is uplink or downlink.
///
/// [quality]: Quality of the network.
///
/// [rtt]: Round-trip time of the network in ms.
///
/// [lostRate]Packet loss rate of the network.
///
/// [bitrate]: Bandwidth of the network in kbps.
///
/// [jitter]: Jitter of the network in ms.
typedef OnNetworkDetectionResultType = void Function(
    NetworkDetectionLinkType type,
    NetworkQuality quality,
    int rtt,
    double lostRate,
    int bitrate,
    int jitter);

/// [reason]: Reason for stopping detection.
typedef OnNetworkDetectionStoppedType = void Function(
    NetworkDetectionStopReason reason);

/// [mixId]: Audio mixing task ID.
///
/// [state]: Mixing status.
///
/// [error]: Mixing related error code.
typedef OnAudioMixingStateChangedType = void Function(
    int mixId, AudioMixingState state, AudioMixingError error);

/// [mixId]: Audio mixing task ID.
///
/// [progress]: The current playback progress of the mixed audio file, in milliseconds.
typedef OnAudioMixingPlayingProgressType = void Function(
    int mixId, int progress);

/// [mode]: Indicates whether publish fallback is enabled locally.
///
/// [roomId]: When publish performance fallback is disabled, roomId is empty. When publish performance fallback is enabled, roomId is the ID of the room affected by the alert.
///
/// [reason]: Reasons for the warning.
///
/// [data]: Performance fallback related data
typedef OnPerformanceAlarmsType = void Function(PerformanceAlarmMode mode,
    String roomId, PerformanceAlarmReason reason, SourceWantedData data);

/// [event]: Information on changes in audio and video streams.
typedef OnSimulcastSubscribeFallbackType = void Function(
    RemoteStreamSwitch event);

/// [state]: Current HTTP proxy connection status.
typedef OnHttpProxyStateType = void Function(int state);

/// [state]: Current HTTPS proxy connection status.
typedef OnHttpsProxyStateType = void Function(int state);

/// [state]: Current SOCKS5 proxy connection status.
typedef OnSocks5ProxyStateType = void Function(int state, String cmd,
    String proxyAddress, String localAddress, String remoteAddress);

/// [type]: Types of recorded streams.
///
/// [state]: Recording status.
///
/// [errorCode]: Recording error code.
///
/// [info]: Details of the recorded file.
typedef OnRecordingStateUpdateType = void Function(StreamIndex type,
    RecordingState state, RecordingErrorCode errorCode, RecordingInfo info);

/// [type]: Types of recorded streams.
///
/// [progress]: Recording progress.
///
/// [info]: Details of the recorded file.
typedef OnRecordingProgressUpdateType = void Function(
    StreamIndex type, RecordingProgress progress, RecordingInfo info);

/// [state]: Recording state.
///
/// [errorCode]: Recording error code.
typedef OnAudioRecordingStateUpdateType = void Function(
    AudioRecordingState state, AudioRecordingErrorCode errorCode);

/// [roomId]: ID of the room where the public stream is published.
///
/// [publicStreamId]: Public stream ID.
///
/// [errorCode]: Publishing result.
typedef OnPushPublicStreamResultType = void Function(
    String roomId, String publicStreamId, PublicStreamErrorCode errorCode);

/// [publicStreamId]: Public stream ID.
///
/// [errorCode]: Subscribing result.
typedef OnPlayPublicStreamResultType = void Function(
    String publicStreamId, PublicStreamErrorCode errorCode);

/// [publicStreamId]: Public stream ID.
///
/// [message]: Content of the received SEI message.
///
/// [sourceType]: SEI message type.
typedef OnPublicStreamDataMessageReceivedType = void Function(
    String publicStreamId, Uint8List message, DataMessageSourceType sourceType);

/// [publicStreamId]: Public stream ID.
///
/// [videoFrameInfo]: Video frame information.
typedef OnFirstPublicStreamVideoFrameDecodedType = void Function(
    String publicStreamId, VideoFrameInfo videoFrameInfo);

/// [publicStreamId]: Public stream ID.
typedef OnFirstPublicStreamAudioFrameType = void Function(
    String publicStreamId);

/// [interval]: Time elapsed from enabling the cloud proxy to successful connection, in ms.
typedef OnCloudProxyConnectedType = void Function(int interval);

/// [result]: Detection result.
typedef OnEchoTestResultType = void Function(EchoTestResult result);

/// [days]: Expiration time in days.
typedef OnLicenseWillExpireType = void Function(int days);

/// @nodoc('For internal use')
/// [param]: Result of callback in JSON string.
typedef OnInvokeExperimentalAPIType = void Function(String param);

/// [result]: Callback that notifies you the result of the echo detection before a call.
typedef OnHardwareEchoDetectionResultType = void Function(
    HardwareEchoDetectionResult result);

/// [localProxyType]: Types of local proxy.
///
/// [localProxyState]: States of local proxy connection.
///
/// [localProxyError]: Errors of local proxy connection.
typedef OnLocalProxyStateChangedType = void Function(
    LocalProxyType localProxyType,
    LocalProxyState localProxyState,
    LocalProxyError localProxyError);

/// RTCVideo event callback
class RTCVideoEventHandler extends RTCVideoEventValue {
  /// Callbacks when warning occurs.
  ///
  /// This callback indicates that a (network or media-related) warning has occurred while the SDK is running. Normally, the warning message can be ignored and the SDK will automatically recover.
  OnWarningType? onWarning;

  /// Callbacks when error occurs.
  ///
  /// This callback is triggered when a network or media related error occurs while the SDK is running and cannot be automatically recovered.
  OnErrorType? onError;

  /// Valid since v3.56.1
  ///
  /// Failed to access the extension.<br>
  /// RTC SDK provides some features with extensions. Without implementing the extension, you cannot use the corresponding feature.
  OnExtensionAccessErrorType? onExtensionAccessError;

  /// Callback for failed room creation.
  OnCreateRoomStateChangedType? onCreateRoomStateChanged;

  /// This callback is triggered when the status of the network connection between the SDK and the RTC server changes.
  OnConnectionStateChanged? onConnectionStateChanged;

  /// This callback is triggered when the client's network connection type has changed.
  OnNetworkTypeChangedType? onNetworkTypeChanged;

  /// When a user calls [RTCVideo.startAudioCapture] to turn on the device for audio capture, others in the room will receive this callback.
  OnUserOperateMediaCaptureType? onUserStartAudioCapture;

  /// When a user calls [RTCVideo.stopAudioCapture] to turn off audio capture, others in the room will receive this callback.
  OnUserOperateMediaCaptureType? onUserStopAudioCapture;

  /// The local user will receive this callback when the first local audio frame is captured.
  OnFirstLocalAudioFrameType? onFirstLocalAudioFrame;

  /// The local user will receive this callback after receiving the first remote audio frame.
  OnFirstRemoteAudioFrameType? onFirstRemoteAudioFrame;

  /// After calling [RTCVideo.enableAudioPropertiesReport], you will periodically receive this callback for the information about local audio.
  ///
  /// Local audio includes the microphone audio, the screen audio captured using RTC SDK internal mechanisms, and locally mixing audio.
  OnLocalAudioPropertiesReportType? onLocalAudioPropertiesReport;

  /// After calling [RTCVideo.enableAudioPropertiesReport], the local user will receive this callback periodically to get the volume information of the remote audio.
  ///
  /// Audio of remote users includes microphone audio and screen audio captured using the RTC SDK's internal capture module.
  OnRemoteAudioPropertiesReportType? onRemoteAudioPropertiesReport;

  /// @nodoc('Not available')
  /// After calling [RTCVideo.enableAudioPropertiesReport], the local user will receive this callback periodically to get information about the most active user in the room.
  OnActiveSpeakerType? onActiveSpeaker;

  /// When a user calls [RTCVideo.startVideoCapture] to start video capture, other users in the room will receive this callback.
  OnUserOperateMediaCaptureType? onUserStartVideoCapture;

  /// When a user calls [RTCVideo.startVideoCapture] to stop video capture, other users in the room will receive this callback.
  OnUserOperateMediaCaptureType? onUserStopVideoCapture;

  /// This callback is triggered when the first locally captured video/screen share screen is rendered in the local view.
  OnFirstLocalVideoFrameCapturedType? onFirstLocalVideoFrameCaptured;

  /// This callback is triggered after the first frame of remote video stream is locally rendered by SDK.
  OnFirstRemoteVideoFrameRenderedType? onFirstRemoteVideoFrameRendered;

  /// This callback after the first frame of remote video stream is received and decoded by SDK.
  ///
  /// + For main stream, after joining the room, the subscriber will receive this callback only when the publisher publishes video stream for the first time.
  /// + For screen-sharing stream, the subscriber will receive this callback every time the publisher publishes or republishes the screen video stream.
  OnFirstRemoteVideoFrameRenderedType? onFirstRemoteVideoFrameDecoded;

  /// Users in the room who subscribe to the video stream receive this callback when the remote video size or rotation information changes.
  OnRemoteVideoSizeChangedType? onRemoteVideoSizeChanged;

  /// This callback when local video size or rotation information changes.
  OnLocalVideoSizeChangedType? onLocalVideoSizeChanged;

  /// Audio device state callback, returns the state of audio capture and audio playback devices.
  OnAudioDeviceStateChangedType? onAudioDeviceStateChanged;

  /// Video device state callback, returns the state of camera capture and screen capture devices.
  OnVideoDeviceStateChangedType? onVideoDeviceStateChanged;

  /// Audio device warning callback. The audio devices includes audio capture devices and audio rendering devices.
  OnAudioDeviceWarningType? onAudioDeviceWarning;

  /// Video device warning callback. The video devices includes video capture devices.
  OnVideoDeviceWarningType? onVideoDeviceWarning;

  /// Callback on first audio frame send status change.
  OnMediaFrameSendStateChangedType? onAudioFrameSendStateChanged;

  /// Callback on first video frame send status change.
  OnMediaFrameSendStateChangedType? onVideoFrameSendStateChanged;

  /// Callback on first screen frame send status change.
  OnMediaFrameSendStateChangedType? onScreenVideoFrameSendStateChanged;

  /// Callback on first audio frame playback status change.
  OnMediaFramePlayStateChangedType? onAudioFramePlayStateChanged;

  /// Callback on first video frame playback status change.
  OnMediaFramePlayStateChangedType? onVideoFramePlayStateChanged;

  /// Callback on first screen frame playback status change.
  OnMediaFramePlayStateChangedType? onScreenVideoFramePlayStateChanged;

  /// This callback is triggered when the audio playback route changes.
  ///
  /// Either of the following action will lead to audio route switching:
  /// + Connecting/Disconnecting an audio device.
  /// + Designating the other audio device by calling [RTCVideo.setAudioRoute].<br>
  /// See [Set the Audio Route](https://docs.byteplus.com/byteplus-rtc/docs/117836) for details.
  OnAudioRouteChangedType? onAudioRouteChanged;

  /// The user receives this callback after receiving a video frame with a SEI message.
  OnSEIMessageReceivedType? onSEIMessageReceived;

  /// Callback about publishing status of the black frame video stream.
  ///
  /// In a voice call scenario, when the local user calls [RTCVideo.sendSEIMessage] to send SEI data with a black frame, the sending status of the stream is notified to the remote user through this callback.<br>
  /// You can tell from this callback that the video frame carrying SEI data is a black frame and thus not render that video frame.
  OnSEIStreamUpdateType? onSEIStreamUpdate;

  /// The user receives this callback after receiving an audio stream sync message from the remote user calling [RTCVideo.sendStreamSyncInfo].
  OnStreamSyncInfoReceivedType? onStreamSyncInfoReceived;

  /// This callback is triggered periodically (2s) reporting the current CPU and memory usage.
  @override
  abstract OnSysStatsType? onSysStats;

  /// The local user receives this callback when the audio state changes.
  @Deprecated(
      'Deprecated since v3.50.1 and will be deleted in v3.56.1, use `RTCVideoEventHandler.onAudioDeviceStateChanged` instead.')
  OnLocalAudioStateChangedType? onLocalAudioStateChanged;

  /// The user receives this callback when the state of the audio stream from the remote end changes.
  /// @nodoc('Not available')
  OnRemoteAudioStateChangedType? onRemoteAudioStateChanged;

  /// The local user receives this callback when the video state changes.
  @Deprecated(
      'Deprecated since v3.50.1 and will be deleted in v3.56.1, use `RTCVideoEventHandler.onVideoDeviceStateChanged` instead.')
  OnLocalVideoStateChangedType? onLocalVideoStateChanged;

  /// When the state of the remote video stream changes, users in the room subscribed to this stream receive this callback.
  /// @nodoc('Not available')
  OnRemoteVideoStateChangedType? onRemoteVideoStateChanged;

  /// @nodoc('Not available')
  OnRemoteVideoSuperResolutionModeChangedType?
      onRemoteVideoSuperResolutionModeChanged;

  /// @nodoc('Not available')
  OnVideoDenoiseModeChangedType? onVideoDenoiseModeChanged;

  /// The user receives this callback after calling [RTCVideo.login].
  OnLoginResultType? onLoginResult;

  /// The user receives this callback after calling [RTCVideo.logout] or when the local user is kicked out because another user logs in with the same UserId.
  OnLogoutType? onLogout;

  /// The user receives this callback after calling [RTCVideo.setServerParams].
  OnServerParamsSetResultType? onServerParamsSetResult;

  /// The user receives this callback after calling [RTCVideo.getPeerOnlineStatus].
  OnGetPeerOnlineStatusType? onGetPeerOnlineStatus;

  /// Local users will receive this callback when they receive a text message from a user outside the room calling [RTCVideo.sendUserMessageOutsideRoom].
  OnMessageReceivedType? onUserMessageReceivedOutsideRoom;

  /// Local users will receive this callback when they receive a binary message from a user outside the room calling [RTCVideo.sendUserBinaryMessageOutsideRoom].
  OnBinaryMessageReceivedType? onUserBinaryMessageReceivedOutsideRoom;

  /// Callback for sending a message to a specified user outside the room.
  OnMessageSendResultType? onUserMessageSendResultOutsideRoom;

  /// This callback is asynchronous. This callback is received after the user calls [RTCVideo.sendServerMessage] or [RTCVideo.sendServerBinaryMessage] to send a message.
  OnServerMessageSendResultType? onServerMessageSendResult;

  /// Callback of pre-call network detection result.
  ///
  /// After [RTCVideo.startNetworkDetection] is successfully called, the user will receive this callback in 3s for the first time and every 2s thereafter.
  OnNetworkDetectionResultType? onNetworkDetectionResult;

  /// Callback for end of pre-call network detection.
  ///
  /// The following will stop detection and trigger this callback once:
  /// + Call [RTCVideo.stopNetworkDetection] to stop network detection.
  /// + The first frame of remote/local audio is received.
  /// + The detection lasts more than 3 mins.
  /// + The entire detection link is disconnected for a certain period of time.
  OnNetworkDetectionStoppedType? onNetworkDetectionStopped;

  /// Callback when audio file playback status changes in audio mixing
  ///
  /// The timing at which this callback will be triggered is as follows:
  /// + When [RTCAudioMixingManager.startAudioMixing] is successfully called.
  /// + When calling [RTCAudioMixingManager.startAudioMixing] repeatedly with the same ID.
  /// + When calling [RTCAudioMixingManager.pauseAudioMixing] to pause audio mixing.
  /// + When calling [RTCAudioMixingManager.resumeAudioMixing] to resume audio mixing.
  /// + When calling [RTCAudioMixingManager.stopAudioMixing] to stop audio mixing.
  /// + When playback ends.
  @Deprecated(
      'Deprecated since v3.54.1, use RTCMediaPlayerEventHandler.onMediaPlayerStateChanged and RTCAudioEffectPlayerEventHandler.onAudioEffectPlayerStateChanged instead')
  OnAudioMixingStateChangedType? onAudioMixingStateChanged;

  /// Callback for playback progress of mixed audio files.
  ///
  /// After calling [RTCMediaPlayer.setProgressInterval] to set the time interval to a value greater than 0, or calling [RTCMediaPlayer.start] to set the time interval in `config` to a value greater than 0, the SDK will trigger the callback according to the set time interval.
  OnAudioMixingPlayingProgressType? onAudioMixingPlayingProgress;

  /// This callback is triggered:
  /// + when device performance is detected as insufficient without publishing performance fallback being enabled using [RTCVideo.setPublishFallbackOption].
  /// + when publishing performance falls back/restores due to device performance/network issues after publishing performance fallback is enabled using [RTCVideo.setPublishFallbackOption].
  OnPerformanceAlarmsType? onPerformanceAlarms;

  /// Once the fallback option is set, fallback or reverting from a Fallback of a media stream will trigger this callback.
  OnSimulcastSubscribeFallbackType? onSimulcastSubscribeFallback;

  /// This callback is triggered when the HTTP proxy connection status changes.
  OnHttpProxyStateType? onHttpProxyState;

  /// This callback is triggered when the HTTPS proxy connection status changes.
  OnHttpsProxyStateType? onHttpsProxyState;

  /// This callback is triggered when the Socks5 proxy connection status changes.
  OnSocks5ProxyStateType? onSocks5ProxyState;

  /// The user will receive this callback after calling [RTCVideo.startFileRecording] or [RTCVideo.stopFileRecording].
  OnRecordingStateUpdateType? onRecordingStateUpdate;

  /// This callback is triggered every second after [RTCVideo.startFileRecording] is called for local recording.
  OnRecordingProgressUpdateType? onRecordingProgressUpdate;

  /// After calling [RTCVideo.startAudioRecording]/[RTCVideo.stopAudioRecording] to change the recording status, you will receive the callback.
  OnAudioRecordingStateUpdateType? onAudioRecordingStateUpdate;

  /// @nodoc('Not available')
  /// Callback of the result of publishing the public stream.
  ///
  /// After [RTCVideo.startPushPublicStream] is called to publish a public stream, the start result is notified to the user via this callback.
  OnPushPublicStreamResultType? onPushPublicStreamResult;

  /// @nodoc('Not available')
  /// Callback of the result of the subscription to the public stream.
  ///
  /// After [RTCVideo.startPlayPublicStream] is called to start subscribing to the public stream, the user will receive the start result and the error during the subscription through this callback.
  OnPlayPublicStreamResultType? onPlayPublicStreamResult;

  /// @nodoc('Not available')
  /// Callback of the SEI carried by the public video stream.
  ///
  /// After [RTCVideo.startPlayPublicStream] is called to start subscribing to the public stream, the SEI messages in the public stream are received via this callback.
  ///
  /// The SEI you can get via this callback is inserted by calling [RTCVideo.sendSEIMessage] in the SDK.
  /// You receive SEI from all the video streams if the SEI messages do not have conflicts. However, if the SEI messages from different video streams have conflicts, you will receive only one of them.
  ///
  /// You also need to listen to [RTCVideoEventHandler.onPublicStreamDataMessageReceived] to receive SEI inserted via Open API in the server.
  OnPublicStreamDataMessageReceivedType? onPublicStreamSEIMessageReceived;

  /// @nodoc('Not available')
  /// Callback on receiving the data message carried by the public video stream.<br>
  /// After calling [RTCVideo.startPlayPublicStream], you will receive this callback if the public stream has an SEI message.
  ///
  /// The data messages carried by the public video stream include:
  /// + SEI inserted by calling the OpenAPI. You will receive SEI from all the video streams if the SEI messages do not have conflicts. However, if the SEI  messages from different video streams have conflicts, you will receive only one of them.
  /// + Media volume indicator. You must enable the callback via the OpenAPI on the server.
  ///
  /// You also need to listen to [RTCVideoEventHandler.onPublicStreamSEIMessageReceived] to receive SEI inserted via API in the client SDK.
  OnPublicStreamDataMessageReceivedType? onPublicStreamDataMessageReceived;

  /// @nodoc('Not available')
  /// Callback for successful decoding of the first video frame of a public stream.
  OnFirstPublicStreamVideoFrameDecodedType?
      onFirstPublicStreamVideoFrameDecoded;

  /// @nodoc('Not available')
  /// Callback for successful decoding of the first audio frame of a public stream.
  OnFirstPublicStreamAudioFrameType? onFirstPublicStreamAudioFrame;

  /// After calling [RTCVideo.startCloudProxy] to enable the cloud proxy, this event is called back when the SDK successfully connects to the cloud proxy server for the first time.
  OnCloudProxyConnectedType? onCloudProxyConnected;

  /// Callback about the call test result.
  ///
  /// The timing when this callback will be triggered is as follows:
  /// + A device-related error occurred during the test.
  /// + After a successful test.
  /// + After stopping the test, provided that the audio/video playback was not received during the test due to non-device reasons.
  OnEchoTestResultType? onEchoTestResult;

  /// After calling [RTCVideo.getNetworkTimeInfo] for the first time, the SDK starts network time synchronization internally. This callback will be triggered when the synchronization is completed.
  EmptyCallbackType? onNetworkTimeSynchronized;

  /// License expiration time reminder
  OnLicenseWillExpireType? onLicenseWillExpire;

  /// @nodoc('For internal use')
  OnInvokeExperimentalAPIType? onInvokeExperimentalAPI;

  /// Callback that notifies you the result of the echo detection before a call.
  ///
  /// Notes:
  /// + This callback notifies you the result of the echo detection by calling [RTCVideo.startHardwareEchoDetection].
  /// + We recommend to call [RTCVideo.stopHardwareEchoDetection] to stop the detection.
  /// + Listen to `detectLeakEcho` in the callback of [RTCVideoEventHandler.onAudioDeviceWarning] for the echo issue during a call.
  OnHardwareEchoDetectionResultType? onHardwareEchoDetectionResult;

  /// Callback on local proxy connection. After calling [RTCVideo.setLocalProxy] to set local proxies, you will receive this callback that informs you of the states of local proxy connection.
  OnLocalProxyStateChangedType? onLocalProxyStateChanged;

  /// @nodoc
  RTCVideoEventHandler({
    this.onWarning,
    this.onError,
    this.onExtensionAccessError,
    this.onCreateRoomStateChanged,
    this.onConnectionStateChanged,
    this.onNetworkTypeChanged,
    this.onUserStartAudioCapture,
    this.onUserStopAudioCapture,
    this.onFirstLocalAudioFrame,
    this.onFirstRemoteAudioFrame,
    this.onLocalAudioPropertiesReport,
    this.onRemoteAudioPropertiesReport,
    this.onActiveSpeaker,
    this.onUserStartVideoCapture,
    this.onUserStopVideoCapture,
    this.onFirstLocalVideoFrameCaptured,
    this.onFirstRemoteVideoFrameRendered,
    this.onFirstRemoteVideoFrameDecoded,
    this.onRemoteVideoSizeChanged,
    this.onLocalVideoSizeChanged,
    this.onAudioDeviceStateChanged,
    this.onVideoDeviceStateChanged,
    this.onAudioDeviceWarning,
    this.onVideoDeviceWarning,
    this.onAudioFrameSendStateChanged,
    this.onVideoFrameSendStateChanged,
    this.onScreenVideoFrameSendStateChanged,
    this.onAudioFramePlayStateChanged,
    this.onVideoFramePlayStateChanged,
    this.onScreenVideoFramePlayStateChanged,
    this.onAudioRouteChanged,
    this.onSEIMessageReceived,
    this.onSEIStreamUpdate,
    this.onStreamSyncInfoReceived,
    OnSysStatsType? onSysStats,
    this.onLocalAudioStateChanged,
    this.onRemoteAudioStateChanged,
    this.onLocalVideoStateChanged,
    this.onRemoteVideoStateChanged,
    this.onRemoteVideoSuperResolutionModeChanged,
    this.onVideoDenoiseModeChanged,
    this.onLoginResult,
    this.onLogout,
    this.onServerParamsSetResult,
    this.onGetPeerOnlineStatus,
    this.onUserMessageReceivedOutsideRoom,
    this.onUserBinaryMessageReceivedOutsideRoom,
    this.onUserMessageSendResultOutsideRoom,
    this.onServerMessageSendResult,
    this.onNetworkDetectionResult,
    this.onNetworkDetectionStopped,
    this.onAudioMixingStateChanged,
    this.onAudioMixingPlayingProgress,
    this.onPerformanceAlarms,
    this.onSimulcastSubscribeFallback,
    this.onHttpProxyState,
    this.onHttpsProxyState,
    this.onSocks5ProxyState,
    this.onRecordingStateUpdate,
    this.onRecordingProgressUpdate,
    this.onAudioRecordingStateUpdate,
    this.onPushPublicStreamResult,
    this.onPlayPublicStreamResult,
    this.onPublicStreamSEIMessageReceived,
    this.onPublicStreamDataMessageReceived,
    this.onFirstPublicStreamVideoFrameDecoded,
    this.onFirstPublicStreamAudioFrame,
    this.onCloudProxyConnected,
    this.onEchoTestResult,
    this.onNetworkTimeSynchronized,
    this.onLicenseWillExpire,
    this.onInvokeExperimentalAPI,
    this.onHardwareEchoDetectionResult,
    this.onLocalProxyStateChanged,
  }) : super(onSysStats: onSysStats);
}
