// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

// ignore_for_file: public_member_api_docs
import '../api/bytertc_asr_engine.dart';

extension RTCASREngineEventProcessor on RTCASREngineEventHandler {
  void process(String methodName, Map<dynamic, dynamic> dic) {
    switch (methodName) {
      case 'onSuccess':
        onSuccess?.call();
        break;
      case 'onMessage':
        final data = OnMessageData.fromMap(dic);
        onMessage?.call(data.message);
        break;
      case 'onError':
        final data = OnErrorMsgData.fromMap(dic);
        onErrorMsg?.call(data.errorCode, data.errorMessage);
        break;
      default:
        break;
    }
  }
}

class OnMessageData {
  final String message;

  const OnMessageData({required this.message});

  factory OnMessageData.fromMap(Map<dynamic, dynamic> map) {
    return OnMessageData(message: map['message']);
  }
}

class OnErrorMsgData {
  final int errorCode;
  final String errorMessage;

  const OnErrorMsgData({required this.errorCode, required this.errorMessage});

  factory OnErrorMsgData.fromMap(Map<dynamic, dynamic> map) {
    return OnErrorMsgData(
        errorCode: map['errorCode'], errorMessage: map['errorMessage']);
  }
}
