// Copyright 2023 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

// ignore_for_file: public_member_api_docs

import '../api/bytertc_cdn_stream.dart';
import 'base/bytertc_enum_convert.dart';

extension RTCMixedStreamProcessor on RTCMixedStreamObserver {
  void process(String methodName, Map<dynamic, dynamic> dic) {
    switch (methodName) {
      case 'onMixingEvent':
        final data = OnMixingEventData.fromMap(dic);
        onMixingEvent?.call(
            data.eventType, data.taskId, data.error, data.mixType);
        break;
      default:
        break;
    }
  }
}

extension RTCPushSingleStreamToCDNProcessor
    on RTCPushSingleStreamToCDNObserver {
  void process(String methodName, Map<dynamic, dynamic> dic) {
    switch (methodName) {
      case 'onStreamPushEvent':
        OnStreamPushEventData data = OnStreamPushEventData.fromMap(dic);
        onStreamPushEvent?.call(data.eventType, data.taskId, data.error);
        break;
      default:
        break;
    }
  }
}

class OnMixingEventData {
  final MixedStreamEvent eventType;
  final String taskId;
  final MixedStreamErrorCode error;
  final MixedStreamType mixType;

  const OnMixingEventData(
      {required this.eventType,
      required this.taskId,
      required this.error,
      required this.mixType});

  factory OnMixingEventData.fromMap(Map<dynamic, dynamic> map) {
    return OnMixingEventData(
        eventType: (map['eventType'] as int).mixedStreamEvent,
        taskId: map['taskId'],
        error: (map['error'] as int).mixedStreamErrorCode,
        mixType: (map['mixType'] as int).mixedStreamType);
  }
}

class OnStreamPushEventData {
  final StreamSinglePushEvent eventType;
  final String taskId;
  final int error;

  const OnStreamPushEventData(
      {required this.eventType, required this.taskId, required this.error});

  factory OnStreamPushEventData.fromMap(Map<dynamic, dynamic> map) {
    return OnStreamPushEventData(
        eventType: (map['eventType'] as int).streamSinglePushEvent,
        taskId: map['taskId'],
        error: map['error']);
  }
}
