// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

// ignore_for_file: public_member_api_docs
import '../api/bytertc_live_transcoding.dart';
import 'base/bytertc_enum_convert.dart';

extension RTCLiveTranscodingProcessor on RTCLiveTranscodingObserver {
  void process(String methodName, Map<dynamic, dynamic> dic) {
    switch (methodName) {
      case 'onStreamMixingEvent':
        final data = OnStreamMixingEventData.fromMap(dic);
        onStreamMixingEvent?.call(
            data.eventType, data.taskId, data.error, data.mixType);
        break;
      default:
        break;
    }
  }
}

class OnStreamMixingEventData {
  final StreamMixingEvent eventType;
  final String taskId;
  final TranscoderErrorCode error;
  final StreamMixingType mixType;

  const OnStreamMixingEventData(
      {required this.eventType,
      required this.taskId,
      required this.error,
      required this.mixType});

  factory OnStreamMixingEventData.fromMap(Map<dynamic, dynamic> map) {
    return OnStreamMixingEventData(
        eventType: (map['eventType'] as int).streamMixingEvent,
        taskId: map['taskId'],
        error: (map['error'] as int).transcoderErrorCode,
        mixType: (map['mixType'] as int).streamMixingType);
  }
}
