// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

part of '../api/bytertc_render_view.dart';

class _RTCSurfaceViewState extends State<RTCSurfaceView> {
  static final Map<int, MethodChannel> _channels = {};

  int? _id;

  @override
  Widget build(BuildContext context) {
    if (defaultTargetPlatform == TargetPlatform.android) {
      var creationParams = videoParams();
      creationParams.addAll({
        'viewType': widget.viewType.index,
        'zOrderOnTop': widget.zOrderOnTop,
        'zOrderMediaOverlay': widget.zOrderMediaOverlay,
      });
      return GestureDetector(
        behavior: HitTestBehavior.opaque,
        child: AndroidView(
          viewType: 'ByteRTCSurfaceView',
          onPlatformViewCreated: _onPlatformViewCreated,
          hitTestBehavior: PlatformViewHitTestBehavior.transparent,
          creationParams: creationParams,
          creationParamsCodec: const StandardMessageCodec(),
        ),
      );
    } else if (defaultTargetPlatform == TargetPlatform.iOS) {
      return GestureDetector(
        behavior: HitTestBehavior.opaque,
        child: UiKitView(
          viewType: 'ByteRTCSurfaceView',
          onPlatformViewCreated: _onPlatformViewCreated,
          hitTestBehavior: PlatformViewHitTestBehavior.transparent,
          creationParams: videoParams(),
          creationParamsCodec: const StandardMessageCodec(),
        ),
      );
    }
    return Text('$defaultTargetPlatform is not yet supported by the plugin');
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  void didUpdateWidget(RTCSurfaceView oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.context.uid != widget.context.uid ||
        oldWidget.context.roomId != widget.context.roomId ||
        oldWidget.context.canvasType != widget.context.canvasType ||
        oldWidget.context.streamType != widget.context.streamType) {
      _setVideoCanvas();
    } else if (oldWidget.renderMode != widget.renderMode ||
        oldWidget.backgroundColor != widget.backgroundColor ||
        oldWidget.renderRotation != widget.renderRotation) {
      _updateVideoCanvas();
    }

    if (defaultTargetPlatform == TargetPlatform.android) {
      if (oldWidget.zOrderOnTop != widget.zOrderOnTop) {
        _setZOrderOnTop();
      }
      if (oldWidget.zOrderMediaOverlay != widget.zOrderMediaOverlay) {
        _setZOrderMediaOverlay();
      }
    }
  }

  @override
  void dispose() {
    super.dispose();
    _channels.remove(_id);
  }

  Map<String, dynamic> videoParams() {
    return <String, dynamic>{
      'canvasType': widget.context.canvasType.index,
      'roomId': widget.context.roomId,
      'uid': widget.context.uid,
      'streamIndex': widget.context.streamType.index,
      'renderMode': widget.renderMode.value,
      'backgroundColor': widget.backgroundColor,
      'renderRotation': widget.renderRotation.value,
    };
  }

  void _setVideoCanvas() {
    _channels[_id]?.invokeMethod('setVideoCanvas', videoParams());
  }

  void _updateVideoCanvas() {
    _channels[_id]?.invokeMethod('updateVideoCanvas', videoParams());
  }

  void _setZOrderOnTop() {
    _channels[_id]?.invokeMethod('setZOrderOnTop', {
      'onTop': widget.zOrderOnTop,
    });
  }

  void _setZOrderMediaOverlay() {
    _channels[_id]?.invokeMethod('setZOrderMediaOverlay', {
      'isMediaOverlay': widget.zOrderMediaOverlay,
    });
  }

  void _onPlatformViewCreated(int id) async {
    _id = id;
    if (!_channels.containsKey(id)) {
      _channels[id] = MethodChannel('com.bytedance.ve_rtc_surfaceView$id');
    }
    widget.onPlatformViewCreated?.call(id);
  }
}
