// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

// ignore_for_file: public_member_api_docs
import '../api/bytertc_event_define.dart';
import '../api/bytertc_media_defines.dart';
import '../api/bytertc_room_event_handler.dart';
import '../api/bytertc_rts_defines.dart';
import '../api/bytertc_video_defines.dart';
import 'base/bytertc_enum_convert.dart';
import 'base/bytertc_event_serialize.dart';

class RTCRoomEventValue {
  RTCRoomEventValue({
    OnRoomStatsType? onRoomStats,
    OnLocalStreamStatsType? onLocalStreamStats,
    OnRemoteStreamStatsType? onRemoteStreamStats,
    OnNetworkQualityType? onNetworkQuality,
  }) {
    this.onRoomStats = onRoomStats;
    this.onLocalStreamStats = onLocalStreamStats;
    this.onRemoteStreamStats = onRemoteStreamStats;
    this.onNetworkQuality = onNetworkQuality;
  }

  void Function(Map<String, dynamic>)? _valueObserver;

  set valueObserver(void Function(Map<String, dynamic>)? valueObserver) {
    _valueObserver = valueObserver;
    if (valueObserver == null) return;
    _valueObserver?.call({
      'enableRoomStats': _onRoomStats != null,
      'enableLocalStreamStats': _onLocalStreamStats != null,
      'enableRemoteStreamStats': _onRemoteStreamStats != null,
      'enableNetworkQualityStats': _onNetworkQuality != null
    });
  }

  OnRoomStatsType? _onRoomStats;

  OnRoomStatsType? get onRoomStats => _onRoomStats;

  set onRoomStats(OnRoomStatsType? onRoomStats) {
    _onRoomStats = onRoomStats;
    _valueObserver?.call({'enableRoomStats': onRoomStats != null});
  }

  OnLocalStreamStatsType? _onLocalStreamStats;

  OnLocalStreamStatsType? get onLocalStreamStats => _onLocalStreamStats;

  set onLocalStreamStats(OnLocalStreamStatsType? onLocalStreamStats) {
    _onLocalStreamStats = onLocalStreamStats;
    _valueObserver
        ?.call({'enableLocalStreamStats': onLocalStreamStats != null});
  }

  OnRemoteStreamStatsType? _onRemoteStreamStats;

  OnRemoteStreamStatsType? get onRemoteStreamStats => _onRemoteStreamStats;

  set onRemoteStreamStats(OnRemoteStreamStatsType? onRemoteStreamStats) {
    _onRemoteStreamStats = onRemoteStreamStats;
    _valueObserver
        ?.call({'enableRemoteStreamStats': onRemoteStreamStats != null});
  }

  OnNetworkQualityType? _onNetworkQuality;

  OnNetworkQualityType? get onNetworkQuality => _onNetworkQuality;

  set onNetworkQuality(OnNetworkQualityType? onNetworkQuality) {
    _onNetworkQuality = onNetworkQuality;
    _valueObserver
        ?.call({'enableNetworkQualityStats': onNetworkQuality != null});
  }
}

extension RTCRoomEventProcessor on RTCRoomEventHandler {
  void process(String methodName, Map<dynamic, dynamic> dic) {
    switch (methodName) {
      case 'onRoomStateChanged':
        final data = OnRoomStateChangedData.fromMap(dic);
        onRoomStateChanged?.call(
            data.roomId, data.uid, data.state, data.extraInfo);
        break;
      case 'onStreamStateChanged':
        final data = OnRoomStateChangedData.fromMap(dic);
        onStreamStateChanged?.call(
            data.roomId, data.uid, data.state, data.extraInfo);
        break;
      case 'onLeaveRoom':
        final data = OnLeaveRoomData.fromMap(dic);
        onLeaveRoom?.call(data.rtcRoomStats);
        break;
      case 'onAVSyncStateChange':
        final data = OnAVSyncStateChangeData.fromMap(dic);
        onAVSyncStateChange?.call(data.state);
        break;
      case 'onRoomStats':
        final data = OnRoomStatsData.fromMap(dic);
        onRoomStats?.call(data.stats);
        break;
      case 'onLocalStreamStats':
        final data = OnLocalStreamStatsData.fromMap(dic);
        onLocalStreamStats?.call(data.stats);
        break;
      case 'onRemoteStreamStats':
        final data = OnRemoteStreamStatsData.fromMap(dic);
        onRemoteStreamStats?.call(data.stats);
        break;
      case 'onUserJoined':
        final data = OnUserJoinedData.fromMap(dic);
        onUserJoined?.call(data.userInfo, data.elapsed);
        break;
      case 'onUserLeave':
        final data = OnUserLeaveData.fromMap(dic);
        onUserLeave?.call(data.uid, data.reason);
        break;
      case 'onTokenWillExpire':
        onTokenWillExpire?.call();
        break;
      case 'onPublishPrivilegeTokenWillExpire':
        onPublishPrivilegeTokenWillExpire?.call();
        break;
      case 'onSubscribePrivilegeTokenWillExpire':
        onSubscribePrivilegeTokenWillExpire?.call();
        break;
      case 'onUserPublishStream':
        final data = OnUserPublishStreamData.fromMap(dic);
        onUserPublishStream?.call(data.uid, data.type);
        break;
      case 'onUserUnpublishStream':
        final data = OnUserUnpublishStreamData.fromMap(dic);
        onUserUnpublishStream?.call(data.uid, data.type, data.reason);
        break;
      case 'onUserPublishScreen':
        final data = OnUserPublishStreamData.fromMap(dic);
        onUserPublishScreen?.call(data.uid, data.type);
        break;
      case 'onUserUnpublishScreen':
        final data = OnUserUnpublishStreamData.fromMap(dic);
        onUserUnpublishScreen?.call(data.uid, data.type, data.reason);
        break;
      case 'onStreamSubscribed':
        final data = OnStreamSubscribedData.fromMap(dic);
        onStreamSubscribed?.call(data.stateCode, data.uid, data.info);
        break;
      case 'onRoomMessageReceived':
        final data = OnMessageReceivedData.fromMap(dic);
        onRoomMessageReceived?.call(data.uid, data.message);
        break;
      case 'onRoomBinaryMessageReceived':
        final data = OnBinaryMessageReceivedData.fromMap(dic);
        onRoomBinaryMessageReceived?.call(data.uid, data.message);
        break;
      case 'onUserMessageReceived':
        final data = OnMessageReceivedData.fromMap(dic);
        onUserMessageReceived?.call(data.uid, data.message);
        break;
      case 'onUserBinaryMessageReceived':
        final data = OnBinaryMessageReceivedData.fromMap(dic);
        onUserBinaryMessageReceived?.call(data.uid, data.message);
        break;
      case 'onUserMessageSendResult':
        final data = OnMessageSendResultData.fromMap(dic);
        onUserMessageSendResult?.call(data.msgid, data.error);
        break;
      case 'onRoomMessageSendResult':
        final data = OnRoomMessageSendResultData.fromMap(dic);
        onRoomMessageSendResult?.call(data.msgid, data.error);
        break;
      case 'onSetRoomExtraInfoResult':
        final data = OnSetRoomExtraInfoResultData.fromMap(dic);
        onSetRoomExtraInfoResult?.call(data.taskId, data.error);
        break;
      case 'onRoomExtraInfoUpdate':
        final data = OnRoomExtraInfoUpdateData.fromMap(dic);
        onRoomExtraInfoUpdate?.call(
            data.key, data.value, data.lastUpdateUserId, data.lastUpdateTimeMs);
        break;
      case 'onUserVisibilityChanged':
        final data = OnUserVisibilityChangedData.fromMap(dic);
        onUserVisibilityChanged?.call(
            data.currentUserVisibility, data.errorCode);
        break;
      case 'onVideoStreamBanned':
        final data = OnVideoStreamBannedData.fromMap(dic);
        onVideoStreamBanned?.call(data.uid, data.banned);
        break;
      case 'onAudioStreamBanned':
        final data = OnAudioStreamBannedData.fromMap(dic);
        onAudioStreamBanned?.call(data.uid, data.banned);
        break;
      case 'onForwardStreamStateChanged':
        final data = OnForwardStreamStateChangedData.fromMap(dic);
        onForwardStreamStateChanged?.call(data.stateInfos);
        break;
      case 'onForwardStreamEvent':
        final data = OnForwardStreamEventData.fromMap(dic);
        onForwardStreamEvent?.call(data.eventInfos);
        break;
      case 'onNetworkQuality':
        final data = OnNetworkQualityData.fromMap(dic);
        onNetworkQuality?.call(data.localQuality, data.remoteQualities);
        break;
      case 'onSubtitleStateChanged':
        final data = OnSubtitleStateChangedData.fromMap(dic);
        onSubtitleStateChanged?.call(
            data.state, data.errorCode, data.errorMessage);
        break;
      case 'onSubtitleMessageReceived':
        final data = OnSubtitleMessageReceivedData.fromMap(dic);
        onSubtitleMessageReceived?.call(data.subtitles);
        break;
      default:
        break;
    }
  }
}

class OnRoomStateChangedData {
  final String roomId;
  final String uid;
  final int state;
  final String extraInfo;

  const OnRoomStateChangedData({
    required this.roomId,
    required this.uid,
    required this.state,
    required this.extraInfo,
  });

  factory OnRoomStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnRoomStateChangedData(
        roomId: map['roomId'] as String,
        uid: map['uid'] as String,
        state: map['state'] as int,
        extraInfo: map['extraInfo'] as String);
  }
}

class OnLeaveRoomData {
  final RTCRoomStats rtcRoomStats;

  const OnLeaveRoomData({required this.rtcRoomStats});

  factory OnLeaveRoomData.fromMap(Map<dynamic, dynamic> map) {
    if (map['stats'] == null) {
      return const OnLeaveRoomData(rtcRoomStats: RTCRoomStats());
    }
    return OnLeaveRoomData(rtcRoomStats: RTCRoomStats.fromMap(map['stats']));
  }
}

class OnAVSyncStateChangeData {
  final AVSyncState state;

  const OnAVSyncStateChangeData({required this.state});

  factory OnAVSyncStateChangeData.fromMap(Map<dynamic, dynamic> map) {
    return OnAVSyncStateChangeData(state: (map['state'] as int).avSyncState);
  }
}

class OnRoomStatsData {
  final RTCRoomStats stats;

  const OnRoomStatsData({required this.stats});

  factory OnRoomStatsData.fromMap(Map<dynamic, dynamic> map) {
    return OnRoomStatsData(stats: RTCRoomStats.fromMap(map['stats']));
  }
}

class OnLocalStreamStatsData {
  final LocalStreamStats stats;

  const OnLocalStreamStatsData({required this.stats});

  factory OnLocalStreamStatsData.fromMap(Map<dynamic, dynamic> map) {
    return OnLocalStreamStatsData(
        stats: LocalStreamStats.fromMap(map['stats']));
  }
}

class OnRemoteStreamStatsData {
  final RemoteStreamStats stats;

  const OnRemoteStreamStatsData({required this.stats});

  factory OnRemoteStreamStatsData.fromMap(Map<dynamic, dynamic> map) {
    return OnRemoteStreamStatsData(
        stats: RemoteStreamStats.fromMap(map['stats']));
  }
}

class OnUserJoinedData {
  final UserInfo userInfo;
  final int elapsed;

  const OnUserJoinedData({required this.userInfo, required this.elapsed});

  factory OnUserJoinedData.fromMap(Map<dynamic, dynamic> map) {
    return OnUserJoinedData(
        userInfo: UserInfo.fromMap(map['userInfo']), elapsed: map['elapsed']);
  }
}

class OnUserLeaveData {
  final String uid;
  final UserOfflineReason reason;

  const OnUserLeaveData({required this.uid, required this.reason});

  factory OnUserLeaveData.fromMap(Map<dynamic, dynamic> map) {
    return OnUserLeaveData(
        uid: map['uid'], reason: (map['reason'] as int).userOfflineReason);
  }
}

class OnUserPublishStreamData {
  final String uid;
  final MediaStreamType type;

  const OnUserPublishStreamData({required this.uid, required this.type});

  factory OnUserPublishStreamData.fromMap(Map<dynamic, dynamic> map) {
    return OnUserPublishStreamData(
        uid: map['uid'], type: (map['type'] as int).mediaStreamType);
  }
}

class OnUserUnpublishStreamData {
  final String uid;
  final MediaStreamType type;
  final StreamRemoveReason reason;

  const OnUserUnpublishStreamData(
      {required this.uid, required this.type, required this.reason});

  factory OnUserUnpublishStreamData.fromMap(Map<dynamic, dynamic> map) {
    return OnUserUnpublishStreamData(
        uid: map['uid'],
        type: (map['type'] as int).mediaStreamType,
        reason: (map['reason'] as int).streamRemoveReason);
  }
}

class OnStreamSubscribedData {
  final SubscribeState stateCode;
  final String uid;
  final SubscribeConfig info;

  const OnStreamSubscribedData(
      {required this.stateCode, required this.uid, required this.info});

  factory OnStreamSubscribedData.fromMap(Map<dynamic, dynamic> map) {
    return OnStreamSubscribedData(
        stateCode: (map['stateCode'] as int).subscribeState,
        uid: map['uid'],
        info: SubscribeConfig.fromMap(map['info']));
  }
}

class OnRoomMessageSendResultData {
  final int msgid;
  final RoomMessageSendResult error;

  const OnRoomMessageSendResultData({required this.msgid, required this.error});

  factory OnRoomMessageSendResultData.fromMap(Map<dynamic, dynamic> map) {
    return OnRoomMessageSendResultData(
        msgid: map['msgid'],
        error: (map['error'] as int).roomMessageSendResult);
  }
}

class OnSetRoomExtraInfoResultData {
  final int taskId;
  final SetRoomExtraInfoResult error;

  const OnSetRoomExtraInfoResultData({
    required this.taskId,
    required this.error,
  });

  factory OnSetRoomExtraInfoResultData.fromMap(Map<dynamic, dynamic> map) {
    return OnSetRoomExtraInfoResultData(
      taskId: map['taskId'],
      error: (map['error'] as int?).setRoomExtraInfoResult,
    );
  }
}

class OnRoomExtraInfoUpdateData {
  final String key;
  final String value;
  final String lastUpdateUserId;
  final int lastUpdateTimeMs;

  const OnRoomExtraInfoUpdateData({
    required this.key,
    required this.value,
    required this.lastUpdateUserId,
    required this.lastUpdateTimeMs,
  });

  factory OnRoomExtraInfoUpdateData.fromMap(Map<dynamic, dynamic> map) {
    return OnRoomExtraInfoUpdateData(
      key: map['key'],
      value: map['value'],
      lastUpdateUserId: map['lastUpdateUserId'],
      lastUpdateTimeMs: map['lastUpdateTimeMs'],
    );
  }
}

class OnUserVisibilityChangedData {
  final bool currentUserVisibility;
  final UserVisibilityChangeError errorCode;

  const OnUserVisibilityChangedData({
    required this.currentUserVisibility,
    required this.errorCode,
  });

  factory OnUserVisibilityChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnUserVisibilityChangedData(
      currentUserVisibility: map['currentUserVisibility'],
      errorCode: (map['errorCode'] as int?).userVisibilityChangeError,
    );
  }
}

class OnVideoStreamBannedData {
  final String uid;
  final bool banned;

  const OnVideoStreamBannedData({required this.uid, required this.banned});

  factory OnVideoStreamBannedData.fromMap(Map<dynamic, dynamic> map) {
    return OnVideoStreamBannedData(uid: map['uid'], banned: map['banned']);
  }
}

class OnAudioStreamBannedData {
  final String uid;
  final bool banned;

  const OnAudioStreamBannedData({required this.uid, required this.banned});

  factory OnAudioStreamBannedData.fromMap(Map<dynamic, dynamic> map) {
    return OnAudioStreamBannedData(uid: map['uid'], banned: map['banned']);
  }
}

class OnForwardStreamStateChangedData {
  final List<ForwardStreamStateInfo> stateInfos;

  const OnForwardStreamStateChangedData({required this.stateInfos});

  factory OnForwardStreamStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnForwardStreamStateChangedData(
        stateInfos: (map['stateInfos'] as List<dynamic>)
            .map((e) => ForwardStreamStateInfo.fromMap(e))
            .toList());
  }
}

class OnForwardStreamEventData {
  final List<ForwardStreamEventInfo> eventInfos;

  const OnForwardStreamEventData({required this.eventInfos});

  factory OnForwardStreamEventData.fromMap(Map<dynamic, dynamic> map) {
    return OnForwardStreamEventData(
        eventInfos: (map['eventInfos'] as List<dynamic>)
            .map((e) => ForwardStreamEventInfo.fromMap(e))
            .toList());
  }
}

class OnNetworkQualityData {
  final NetworkQualityStats localQuality;
  final List<NetworkQualityStats> remoteQualities;

  const OnNetworkQualityData(
      {required this.localQuality, required this.remoteQualities});

  factory OnNetworkQualityData.fromMap(Map<dynamic, dynamic> map) {
    return OnNetworkQualityData(
        localQuality: NetworkQualityStats.fromMap(map['localQuality']),
        remoteQualities: (map['remoteQualities'] as List<dynamic>)
            .map((e) => NetworkQualityStats.fromMap(e))
            .toList());
  }
}

class OnSubtitleStateChangedData {
  final SubtitleState state;
  final SubtitleErrorCode errorCode;
  final String errorMessage;

  const OnSubtitleStateChangedData({
    required this.state,
    required this.errorCode,
    required this.errorMessage,
  });

  factory OnSubtitleStateChangedData.fromMap(Map<dynamic, dynamic> map) {
    return OnSubtitleStateChangedData(
      state: (map['state'] as int?).subtitleState,
      errorCode: (map['errorCode'] as int?).subtitleErrorCode,
      errorMessage: map['errorMessage'],
    );
  }
}

class OnSubtitleMessageReceivedData {
  final List<SubtitleMessage> subtitles;

  const OnSubtitleMessageReceivedData({
    required this.subtitles,
  });

  factory OnSubtitleMessageReceivedData.fromMap(Map<dynamic, dynamic> map) {
    return OnSubtitleMessageReceivedData(
      subtitles: (map['subtitles'] as List)
          .map((e) => SubtitleMessage.fromMap(e))
          .toList(growable: false),
    );
  }
}
